var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { computed } from '@ember-decorators/object';
import { getOwner } from '@ember/application';
import Service from '@ember/service';
export default class BestLanguage extends Service {
    get fastboot() {
        return getOwner(this).lookup('service:fastboot');
    }
    bestLanguage(languages) {
        const supportedBaseLanguages = languages.map(language => {
            return this.getBaseLanguage(language);
        });
        const userLanguages = (this.fastboot && this.fastboot.isFastBoot) || false
            ? this.fetchHeaderLanguages()
            : this.fetchBrowserLanguages();
        const supportedUserLanguages = this.intersectLanguages(userLanguages, supportedBaseLanguages);
        const sortedLanguages = this.sortLanguagesByScore(supportedUserLanguages);
        return sortedLanguages[0] || null;
    }
    bestLanguageOrFirst(languages) {
        const bestLanguage = this.bestLanguage(languages);
        if (bestLanguage)
            return bestLanguage;
        return {
            baseLanguage: this.getBaseLanguage(languages[0]),
            language: languages[0],
            score: 0
        };
    }
    fetchHeaderLanguages() {
        const headers = this.fastboot.request.headers;
        return this.parseHeader(headers.get('Accept-Language') || '');
    }
    fetchBrowserLanguages() {
        const languages = [
            ...navigator.languages,
            navigator.language,
            navigator.userLanguage
        ];
        return Array.from(new Set(languages))
            .filter(language => !!language)
            .map((language, index, array) => ({
            language,
            baseLanguage: this.getBaseLanguage(language),
            score: this.computeScore(index, array.length)
        }));
    }
    parseHeader(header) {
        return header
            .split(',')
            .map(headerItem => this.parseHeaderItem(headerItem));
    }
    parseHeaderItem(item) {
        const [language, scoreString = 'q=1.0'] = item.split(';');
        const score = parseFloat(scoreString.split('=')[1]) || 0;
        return {
            language,
            score,
            baseLanguage: this.getBaseLanguage(language)
        };
    }
    getBaseLanguage(language) {
        return language.replace(/[\-_].+$/, '');
    }
    intersectLanguages(userLanguages, supportedBaseLanguages) {
        return userLanguages.filter(({ baseLanguage }) => {
            return supportedBaseLanguages.includes(baseLanguage);
        });
    }
    sortLanguagesByScore(languages) {
        return languages.sort(({ score: scoreA }, { score: scoreB }) => {
            return scoreB - scoreA;
        });
    }
    computeScore(index, total) {
        const scoreDefaultDivider = 10;
        const score = total <= scoreDefaultDivider
            ? 1 - index / scoreDefaultDivider
            : 1 - index / total;
        return parseFloat(score.toFixed(2));
    }
}
__decorate([
    computed
], BestLanguage.prototype, "fastboot", null);
